/******************************************************************************
 * linear_fit.c
 * Federico Bareilles
 * 26/07/2003
 *
 *
 *****************************************************************************/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "tipper_red.h"
#include "matrix.h"
#include "minimos.h"

/****************************************************************************/
double mi_modelo2(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);

  return c * exp(-t * x);
}

double mi_deriv_c2(double x, double *xx)
{
  //double c = *(xx+0);
  double t = *(xx+1);

  return exp(-t * x);
}

double mi_deriv_t2(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);

  return -1.0 * x * c * exp(-t * x);
}
/****************************************************************************/

struct fit *linear_matrix_fit(float *xd, float *yd, float *sd,
			      int num, float Ta, float Tl)
{
  int i;
  struct fit *f;
  double x[4];
  double xdata[num], ydata[num], sigma[num];
  struct model *m;

  f = (struct fit *) malloc(sizeof(struct fit));
  
  for(i=0; i<num;i++){
    xdata[i] = (double) xd[i];
    ydata[i] = (double) yd[i];
    sigma[i] = (double) sd[i];
  }
  
  m = make_model(mi_modelo2);
  model_add_deriv(m, mi_deriv_c2);
  model_add_deriv(m, mi_deriv_t2);

  m->xdata = xdata;
  m->ydata = ydata;
  m->sigma = sigma;
  m->size = num;

  x[0] = .52 * 290.;   //c; ng = 0.516732 (campana burek);
  x[1] = .3;    //t .2

  if( fit_model(m,x) == 0 ){
    //printf("# Tau: %f, gnTa: %f\n",x[1],x[0]);
    f->tau = x[1];
    f->sigma_tau = sqrt(m->VCV->M[1][1]);
    f->sigma_b = sqrt(m->VCV->M[0][0]);
    f->b = x[0]; /* o... log(x[0]); */
    f->resid = f->chi2f = 0.0;
    for(i=0;i<num;i++){
      f->resid += SQR(m->ydata[i] - mi_modelo2(m->xdata[i], x) );
      f->chi2f += SQR((m->ydata[i] - mi_modelo2(m->xdata[i], x))/m->sigma[i]);
      //printf("# %f %f %f %f\n",xdata[i], ydata[i],m->sigma[i],mi_modelo2(m->xdata[i], x) );
    }
    f->resid = sqrt(f->resid);
  }else{
    free(f);
    f = NULL;
  }

  free(m);
  return f;
}

/****************************************************************************/

#if 0
struct fit *linear_fit(float *x, float *y, float *sigma, int num)
{
  /* y = tau * x + b */
  int i;
  //float w;
  float sx=0,sy=0,sxy=0,sx2=0,sy2=0;//,sw=0;
  struct fit *f;

  f = (struct fit *) malloc(sizeof(struct fit));
  for(i=0;i<num;i++){
    //w    = 1.0/SQR(sigma[i]);
    //sw  += w;
    sx  += x[i];
    sy  += y[i];
    sxy += x[i]*y[i];
    sx2 += SQR(x[i]);
    sy2 += SQR(y[i]);
  }

  f->tau = (num*sxy-sx*sy)/(num*sx2-SQR(sx));
  f->b = (sy-f->tau*sx)/num;

  f->resid = 0.0;
  for(i=0;i<num;i++){
    f->resid += SQR(y[i] - f->tau*x[i] - f->b );
  }
  f->resid = sqrt(f->resid);

  //f->sigma_tau = sqrt( (sy2-SQR(sy)/num)/num);
  f->sigma_tau = -1.0;

  f->sigma_b = f->chi2f = -1.0;

  return f;
}
#endif

#if 1
struct fit *linear_fit(float *x_o, float *y, float *sigma, 
		       int num, float Ta, float Tl)
{
  /* y = tau * x + b */
  int i;
  float w;
  float sx=0.0,sy=0.0,sw=0.0;
  float st2 = 0.0, t;
  float x[num];
  struct fit *f;

  f = (struct fit *) malloc(sizeof(struct fit));
  for(i=0;i<num;i++){
    /* gran moco: */
    x[i] = -1.0 * x_o[i];

    w    = 1.0/SQR(sigma[i]);
    sw  += w;
    sx  += x[i]*w;
    sy  += y[i]*w;
  }
  f->tau = 0.0;
  for(i=0;i<num;i++){
    t = (x[i]-(sx/sw))/sigma[i]; /* Ojo: (x[i]-sx/sw)/sigma[i]; en el orig */
    st2 += SQR(t);
    f->tau += t * y[i]/sigma[i];
  }
  f->tau = f->tau/st2;
  f->b =(sy-sx*f->tau)/sw;
  f->sigma_tau = sqrt(1.0/st2);
  f->sigma_b = sqrt( (1.0+SQR(sx)/(sw*st2))/sw);

  f->resid = 0.0;
  f->chi2f = 0.0;
  for(i=0;i<num;i++){
    f->resid += SQR(y[i] - f->tau*x[i] - f->b );
    f->chi2f += SQR((y[i] - f->b - f->tau * x[i])/sigma[i]);
  }
  f->resid = sqrt(f->resid);

  return f;
}
#endif

